#include "parser.h"
#include "iostream"
#include <QtNetwork/QtNetwork>
#include <QTextCodec>
#include "tnotify.h"
#include "singleton.h"
#include "tthreadlist.h"
#include "tthreadlistitem.h"

parser::parser(QWidget *parent) :
    QWidget(parent),
    url(new QLineEdit()),
    addButton (new QPushButton("Add Thread")),
    deleteButton (new QPushButton("Delete Thread")),
    startAllButton (new QPushButton("Start All")),
    startButton (new QPushButton("Start")),
    ondemandButton(new QPushButton("On Demand")),
    stopButton (new QPushButton("Stop")),
    testButton (new QPushButton("Notify test")),
    listView (new TThreadList()),
    grid (new QGridLayout),
    nm (new QNetworkAccessManager()),
    notifications (&Singleton<TNotify>::Instance()),
    notificationNum(0)
{
    grid->addWidget(url, 1,1);
    grid->addWidget(addButton, 1,2);
    url->setPlaceholderText("Type here thread URL");
    grid->addWidget(startAllButton, 2,2);
    grid->addWidget(startButton, 3,2);
    //grid->addWidget(ondemandButton, 4,2);
    ondemandButton->setDisabled(true);
    grid->addWidget(stopButton, 4,2);
    //stopButton->setEnabled(false);
    grid->addWidget(testButton, 5,2);
    grid->addWidget(deleteButton, 6,2);
    grid->addWidget(listView, 2,1,7,1);
    QSettings settings;
    settings.beginGroup("Urls");
    QStringList saved;
    saved = settings.value("UrlsList").toStringList();
    settings.endGroup();
    for (int i=0; i< saved.count(); i++){
        listView->addThread(QUrl(saved.at(i)));
    }

    this->setLayout(grid);
    listView->setFocus();
    QObject::connect(addButton, SIGNAL(clicked()), this, SLOT(addURL()));
    QObject::connect(url, SIGNAL(returnPressed()), this, SLOT(addURL()));
    QObject::connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteURL()));
    QObject::connect(startButton, SIGNAL(clicked()), this, SLOT(start()));
    QObject::connect(startAllButton, SIGNAL(clicked()), this, SLOT(startAll()));
    QObject::connect(stopButton, SIGNAL(clicked()), this, SLOT(stop()));
    QObject::connect(testButton, SIGNAL(clicked()), this, SLOT(test()));

    this->resize(600,250);

}


void parser::addURL(void){
    //QRegExp* regexp = new QRegExp(QString("(http://dobrochan.ru/[lor|b]/res/[0-9]*.xhtml)"));
    QRegExp* regexp = new QRegExp(QString("(http://dobrochan.ru/(mad|lor|b|vg)/res/([0-9]*).xhtml)"));
    if(QUrl(url->text()).isValid() and regexp->indexIn(url->text()) != -1){
        std::cout<< "Echo 1" << std::endl;
        listView->addThread(QUrl(regexp->cap(1)));
        url->clear();
        std::cout<< "Echo End"<<std::endl;
    }else{
        QMessageBox* msgBox = new QMessageBox();
        msgBox->setText("Invalid URL is specified");
        msgBox->show();
    }
}

void parser::deleteURL(void){
    for(int i=listView->topLevelItemCount()-1; i>= 0; i--){
      if(listView->topLevelItem(i)->isSelected()){
          //std::cout << "Deleting thread: " << urlList.at(i)->getName().toStdString() << std::endl;
          listView->deleteThread(i);
      }
    }
}

void parser::start(void){
    if (!listView->selectedItems().isEmpty()){
        listView->startWatch(listView->currentIndex().row());
    }
}

void parser::startAll(void){
    for(int i=0; i < listView->topLevelItemCount(); i++){
        listView->startWatch(i);
    }
}

void parser::stop(){
    if (!listView->selectedItems().isEmpty()){
        listView->stopWatch(listView->currentIndex().row());
    }
}

QString parser::getTitleFromReply(QString data){
    QRegExp expTitle("\"title\":\\s\"([^\"]*)\"");
    expTitle.indexIn(data);
    QString title = expTitle.cap(1);
    return toUnicode(expTitle.cap(1));
}

QString parser::getMessageFromReply(QString data){
    QRegExp messageExp("\"message\":\\s\"([^\"]*)\"");
    messageExp.indexIn(data);
    QString tmp(messageExp.cap(1));
    tmp = parser::toUnicode(tmp);
    return tmp;
}


QString parser::toUnicode (QString str){
    QRegExp newLine("\\\\r\\\\n");
    str.replace(newLine, "\n");
    QRegExp uni("\\\\u[0-9a-f]{4}");
    int pos = 0;
    while ((pos = uni.indexIn(str, pos)) != -1) {
        str.replace(pos++, 6, QChar(uni.cap(0).right(4).toUShort(0, 16)));
    }
    return str;
}

void parser::test(void){
    notifications->newNotify("mah parser", "asdfsdf");
}


void parser::addNotify(void){
    notificationNum++;
}

void parser::deleteNotify(void){
    if(notificationNum > 0){
        notificationNum--;
    }else{
        std::cout << "Sorry, but current notification number is zero" << std::endl;
    }
}
